<?php
/**
* 2007-2024 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2024 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

define('BOLD_VERSION', '1.2.0');
define('BOLD_ENV', 'PROD');
define('ROOT_URL', dirname(__FILE__));

include_once  'services/BoldLog.php';
include_once  'services/model/Constants.php';
include_once  'services/settings/Settings.php';
include_once  'services/api/BoldAPI.php';
include_once  'services/checkout/BoldCheckout.php';
include_once  'controllers/back/BoldWebhooksController.php';

class Bold extends PaymentModule
{
    protected $config_form = false;
    public $context;
    public $bold_api_client;

    public function __construct()
    {
        $this->name = BoldModule::MODULE_NAME;
        $this->tab = 'payments_gateways';
        $this->version = BOLD_VERSION;
        $this->author = 'Bold';
        $this->need_instance = 1;

        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Bold');
        $this->description = $this->l('Lleva tus ventas a otro nivel con la mejor experiencia de pagos en línea. Recibe pagos con tarjeta de crédito, débito o transferencia bancaria.');
        $this->confirmUninstall = $this->l('Recuerda que al hacer esto, dejarás de recibir pagos con Bold y las configuraciones se perderán. Para volver a usarlo, tendrás que reinstalar el módulo.');
        $this->limited_countries = array('CO');
        $this->limited_currencies = array('COP');
        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
        $this->icon = 'logo_bold.jpg';
        $this->checkout = new BoldCheckout();
        $this->webhooks = new BoldWebhooksController();
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        if (extension_loaded('curl') == false) {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module');
            return false;
        }

        $iso_code = Country::getIsoById(Configuration::get('PS_COUNTRY_DEFAULT'));

        if (in_array($iso_code, $this->limited_countries) == false) {
            $this->_errors[] = $this->l('This module is not available in your country');
            return false;
        }

        include(dirname(__FILE__) . '/sql/install.php');

        return parent::install() &&
            $this->createPaymentStates() &&
            $this->registerHook('header') &&
            $this->registerHook('displayBackOfficeHeader') &&
            $this->registerHook('payment') &&
            $this->registerHook('paymentReturn') &&
            $this->registerHook('paymentOptions') &&
            $this->registerHook('displayTopColumn')&&
            $this->registerHook('displayWrapperTop');
    }

    public function uninstall()
    {
        BoldLog::generate("Desinstalando plugin de Bold");

        include(dirname(__FILE__) . '/sql/uninstall.php');

        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */

    protected function isValidPrefix($from_config=false)
    {
        $prefix = $from_config ? Configuration::get('BOLD_PREFIX') : Tools::getValue('BOLD_PREFIX') ;

        if (!empty($prefix) && !preg_match('/^[a-zA-Z0-9_-]+$/', $prefix)) {
            return false;
        }

        // Verificar si el string no contiene la palabra "test"
        if (strpos($prefix, 'test') !== false) {
            return false;
        }

        return true;
    }
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
        $webhook_url = $this->context->link->getModuleLink($this->name, 'webhook',array(), true);
        $error_message = '';
        $success_message = '';
        if (!$this->isValidPrefix()) {
            $error_message = 'El prefijo contiene caracteres inválidos';
        }else if((bool)Tools::isSubmit('submitBoldModule')){
            $error_message = $this->webhooks->isWebhookConfigured($webhook_url, Tools::getValue('BOLD_API_KEY'));
            if (is_string($error_message) && strlen($error_message)==0) {
                $this->processForm();
                $success_message = 'Haz guardado y configurado los datos correctamente.';
            }
        }

        $this->context->smarty->assign(
            array(
                'boldco_module_dir' => $this->_path,
                'boldco_log' => BoldLog::getLogUrl(),
                'boldco_fields_value' => BoldSettings::getConfigFormValues(),
                'boldco_error_message' => $error_message,
                'boldco_success_message' => $success_message,
                'boldco_webhook_url' => $webhook_url,
            )
        );

        return $this->context->smarty->fetch($this->local_path . 'views/templates/admin/module_configuration.tpl');
    }

    /**
     * Save form data.
     */
    protected function processForm()
    {
        $form_values = BoldSettings::getConfigFormValues();

        foreach (array_keys($form_values) as $configKey) {
            Configuration::updateValue($configKey, Tools::getValue($configKey));
        }
    }

    public function createPaymentStates()
    {
        $order_states = array(
            array('#ccfbff', $this->l('Transacción en proceso'), 'PROCESSING', '100010000'),
            array('#c9fecd', $this->l('Transacción completada'), 'APPROVED', '100010010'),
            array('#fec9c9', $this->l('Transacción rechazada'), 'REJECTED', '100010000'),
            array('#fffb96', $this->l('Transacción pendiente  PSE'), 'PENDING', '100010000'),
            array('#ffb0d9', $this->l('Transacción fallida'), 'FAILED', '100010000'),
        );

        foreach ($order_states as $key => $value) {
            if ($this->orderStateAvailable(Configuration::get('BOLD_STATUS_' . $key)) == 1) {
                continue;
            }

            $order_state = new OrderState();
            $order_state->name = array();
            $order_state->template = array();
            $order_state->module_name = $this->name;
            $order_state->color = $value[0];
            $order_state->invoice = $value[3][0];
            $order_state->send_email = $value[3][1];
            $order_state->unremovable = $value[3][2];
            $order_state->hidden = $value[3][3];
            $order_state->logable = $value[3][4];
            $order_state->delivery = $value[3][5];
            $order_state->shipped = $value[3][6];
            $order_state->paid = $value[3][7];
            $order_state->deleted = $value[3][8];

            $order_state->name = array_fill(0, 10, $value[1]);
            $order_state->template = array_fill(0, 10, $value[2]);

            if ($order_state->add()) {
                $file = _PS_ROOT_DIR_ . '/img/os/' . (int) $order_state->id . '.gif';
                copy((dirname(__FILE__) . '/views/img/bold_logo_light_icon.svg'), $file);
                Configuration::updateValue('BOLD_STATUS_' . $key, $order_state->id);
            }
        }

        return true;
    }

    public static function orderStateAvailable($id_order_state)
    {
        $query = "SELECT COUNT(*) AS count_state FROM " . _DB_PREFIX_ . "order_state
            WHERE id_order_state = '" . pSQL($id_order_state) . "'";
        $result = Db::getInstance()->getRow($query);
        return $result['count_state'];
    }

    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookDisplayBackOfficeHeader()
    {
        if (Tools::getValue('configure') == $this->name) {
            $this->context->controller->addJS($this->_path . 'views/js/back.js');
            $this->context->controller->addCSS($this->_path . 'views/css/common.css');
            $this->context->controller->addCSS($this->_path . 'views/css/back.css');
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path . '/views/js/front.js');
        $this->context->controller->addCSS($this->_path . '/views/css/common.css');
        $this->context->controller->addCSS($this->_path . '/views/css/front.css');
    }

    /**
     * This method is used to render the payment button,
     * Take care if the button should be displayed or not.
     */
    public function hookPayment($params)
    {
        $currency_id = $params['cart']->id_currency;
        $currency = new Currency((int)$currency_id);

        if (in_array($currency->iso_code, $this->limited_currencies) == false)
            return false;

        $this->smarty->assign('module_dir', $this->_path);

        return $this->display(__FILE__, 'views/templates/hook/payment.tpl');
    }

    /**
     * This hook is used to display the order confirmation page.
     */

    public function hookPaymentReturn($params)
    {

    }


    /**
     * Return payment options available for PS 1.7+
     *
     * @param array Hook parameters
     *
     * @return array|null
     */
    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }

        if (!$this->checkCurrency($params['cart'])) {
            return;
        }
        if (!$this->isValidPrefix(true)) {
            return;
        }
        if (!BoldSettings::isConfigFilled()) {
            return;
        }

        BoldLog::generate($this->context->link->getModuleLink($this->name, 'webhook',array(), true));

        $option = new \PrestaShop\PrestaShop\Core\Payment\PaymentOption();

        $checkoutAdditionalInfoTemplate = $this->context->smarty->assign(
            array(
                'boldco_module_dir' => $this->_path,
                'boldco_is_sandbox' => Configuration::get('BOLD_TEST_MODE')
            )
        )->fetch('module:' . BoldModule::MODULE_NAME . '/views/templates/hook/checkout_additional_info.tpl');

        $option->setAdditionalInformation($checkoutAdditionalInfoTemplate)
            ->setCallToActionText($this->l('Paga con'))
            ->setLogo(_MODULE_DIR_ . 'bold/views/img/'.$this->getBoldLogo())
            ->setAction($this->context->link->getModuleLink($this->name, 'preprocess', array(), true));

        return [ $option ];
    }

    public function getBoldLogo(){
        $is_light = Configuration::get('BOLD_BRAND');
        return $is_light ? 'bold_logo_light_icon.svg' : 'bold_logo_dark_icon.svg' ;
    }
    public function checkCurrency($cart)
    {


        $currency_order = new Currency($cart->id_currency);

        if (!in_array($currency_order->iso_code, $this->limited_currencies))
            return false;

        $currencies_module = $this->getCurrency($cart->id_currency);
        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    public function hookDisplayWrapperTop()
    {
        return $this->getDisplayFailure();
    }

    /**
     * @return mixed
     */
    public function getDisplayFailure()
    {
        if (Tools::getValue('typeReturn') == 'bold-failure') {
            $cookie = $this->context->cookie;
            if ($cookie->__isset('error_message')) {
                $this->context->smarty->assign(array('error_message' => $cookie->__get('error_message')));
                $cookie->__unset('error_message');
            }
            return $this->display(__FILE__, 'views/templates/hook/failure.tpl');
        }
    }

    public function hookDisplayTopColumn()
    {
        return $this->getDisplayFailure();
    }
}
